        PROGRAM mie
        IMPLICIT NONE
C
C-------Mie computations for a size distribution 
C       of homogeneous spheres.
c
C==========================================================
C--Ref : Toon and Ackerman, Applied Optics, 1981
C        Stephens, CSIRO, 1979
C Attention : surdimensionement des tableaux
C to be compiled with double precision option (-r8 on Sun)
C AUTHOR: Olivier Boucher
C-------SIZE distribution properties----------------
C--sigma_g : geometric standard deviation 
C--r_0     : geometric number mean radius (um)/modal radius
C--Ntot    : total concentration in m-3 
c
        REAL rmin, rmax    !----integral bounds in  m
        PARAMETER (rmin=0.002E-6, rmax=30.E-6)
c
c--Nmode= 1 mode for nitrate
c--NDis=1 distribution per mode
        INTEGER Nmode, Ndis, mode, dis
        PARAMETER (Nmode=2, Ndis=1)
        REAL sigma_g(Ndis,Nmode), r_0(Ndis,Nmode), Ntot(Ndis,Nmode)
c--Accumulation Soluble (AS)
        DATA r_0    /0.433E-6,0.1E-6/   !--meters
        DATA sigma_g/2.0,1.8/
        DATA Ntot   /1.0,1.0/
        CHARACTER*33 chmode(Nmode)
        DATA chmode/'Nitrate Coarse Soluble (CS)',
     .              'Nitrate Accumulation Soluble (AS)'/
c
        REAL masse,volume,surface,rho
        PARAMETER (rho=1.725E3)  !--dry density nitrate kg/m3
c
c---------- RH growth parameters----------------
c
        INTEGER rh_int,IRH
        PARAMETER(rh_int=12)
        REAL RH_tab(rh_int),RH, rh_dummy
        DATA RH_tab/0.,10.,20,30.,40.,50.,60.,70.,80.,85.,90.,95./
        REAL rwet
c
        REAL growth(rh_int,Nmode)
        DATA growth/
     .       1.0,1.0,1.0,1.05,1.10,1.15,1.21,1.33,1.50,1.65,1.85,1.90, !--CS
     .       1.0,1.0,1.0,1.05,1.10,1.15,1.21,1.33,1.50,1.65,1.85,1.90/ !--AS
c
        REAL n_r, n_i
c
c-------------------------------------
c
        COMPLEX m          !----refractive index m=n_r-i*n_i
        INTEGER Nmax,Nstart !--number of iterations
        COMPLEX k2, k3, z1, z2
        COMPLEX u1,u5,u6,u8
        COMPLEX a(1:21000), b(1:21000)
        COMPLEX I 
        INTEGER n  !--loop index
        REAL pi, nnn
        COMPLEX nn
        REAL Q_ext, Q_abs, Q_sca, g, omega   !--parameters for radius r
        REAL x  !--size parameter
        REAL r  !--radius
        REAL sigma_sca, sigma_ext, sigma_abs
        REAL omegatot,  gtot !--averaged parameters
        COMPLEX ksiz2(-1:21000), psiz2(1:21000)
        COMPLEX nu1z1(1:21010), nu1z2(1:21010)
        COMPLEX nu3z2(0:21000)
        REAL number, deltar
        INTEGER bin, Nbin, k
        PARAMETER (Nbin=700)
c
C---wavelengths STREAMER
        INTEGER Nwv, NwvmaxSW
        PARAMETER (NwvmaxSW=25)
        REAL lambda(1:NwvmaxSW)
        DATA lambda/0.28E-6, 0.30E-6, 0.33E-6, 0.36E-6, 0.40E-6,
     .              0.44E-6, 0.48E-6, 0.52E-6, 0.57E-6, 0.64E-6,
     .              0.69E-6, 0.75E-6, 0.78E-6, 0.87E-6, 1.00E-6,
     .              1.10E-6, 1.19E-6, 1.28E-6, 1.53E-6, 1.64E-6,
     .              2.13E-6, 2.38E-6, 2.91E-6, 3.42E-6, 4.00E-6/
c
        INTEGER nb, nb_lambda
        PARAMETER (nb_lambda=5)
        REAL lambda_ref(nb_lambda)
        DATA lambda_ref /0.443E-6,0.550E-6,0.670E-6,0.765E-6,0.865E-6/ 
c
C---TOA fluxes - Streamer Cs
        REAL weight(1:NwvmaxSW-1)
c        DATA weight/0.839920E1, 0.231208E2, 0.322393E2, 0.465058E2,
c     .              0.678199E2, 0.798964E2, 0.771359E2, 0.888472E2,
c     .              0.115281E3, 0.727565E2, 0.816992E2, 0.336172E2,
c     .              0.914603E2, 0.112706E3, 0.658840E2, 0.524470E2,
c     .              0.391067E2, 0.883864E2, 0.276672E2, 0.681812E2,
c     .              0.190966E2, 0.250766E2, 0.128704E2, 0.698720E1/
C---TOA fluxes - Tad
c        DATA weight/ 4.20, 11.56, 16.12, 23.25, 33.91, 39.95, 38.57,
c     .              44.42, 57.64, 29.36, 47.87, 16.81, 45.74, 56.35,
c     .              32.94, 26.22, 19.55, 44.19, 13.83, 34.09,  9.55,
c      .              12.54,  6.44,  3.49/
C---BOA fluxes - Tad
        DATA weight/ 0.01,  4.05, 9.51,  15.99, 26.07, 33.10, 33.07,
     .              39.91, 52.67, 27.89, 43.60, 13.67, 42.22, 40.12,
     .              32.70, 14.44, 19.48, 14.23, 13.43, 16.42,  8.33,
     .               0.95,  0.65, 2.76/
c
        REAL lambda_int(1:NwvmaxSW-1+nb_lambda)
c
        REAL final_a(1:NwvmaxSW-1+nb_lambda)
        REAL final_g(1:NwvmaxSW-1+nb_lambda)
        REAL final_w(1:NwvmaxSW-1+nb_lambda)
c
        INTEGER band, NbandSW, NbandLW
        PARAMETER (NbandSW=6, NbandLW=5)
c
        REAL gcm_a(NbandSW+NbandLW)
        REAL gcm_g(NbandSW+NbandLW)
        REAL gcm_w(NbandSW+NbandLW)
        REAL gcm_weight_a(NbandSW+NbandLW)
        REAL gcm_weight_g(NbandSW+NbandLW)
        REAL gcm_weight_w(NbandSW+NbandLW)
c
        REAL ss_a(NbandSW+NbandLW+nb_lambda,rh_int)
        REAL ss_w(NbandSW+NbandLW+nb_lambda,rh_int)
        REAL ss_g(NbandSW+NbandLW+nb_lambda,rh_int)
c
        INTEGER NwvmaxLW
        PARAMETER (NwvmaxLW=100)
        REAL Tb, Planck
        PARAMETER (Tb=260.0)
c
        INTEGER wv, nb_wv, nb_wv_i
        PARAMETER (nb_wv=200)
        REAL wv_nitrate(1:nb_wv)
        REAL index_r_nitrate(RH_int,Nmode,1:nb_wv)
        REAL index_i_nitrate(RH_int,Nmode,1:nb_wv)
        REAL count_n_r, count_n_i
c
        pi=4.*ATAN(1.)
        I=CMPLX(0.,1.)
c
c------opening output files
c
       OPEN (unit=14, file='SEXT_nitrate_soluble_6bands.txt')
       OPEN (unit=15, file='SSA_nitrate_soluble_6bands.txt')
       OPEN (unit=16, file='G_nitrate_soluble_6bands.txt')
c
       OPEN (unit=34, file='SEXT_nitrate_soluble_5wave.txt')
       OPEN (unit=35, file='SEXT_nitrate_insoluble_5wave.txt')
       OPEN (unit=36, file='SABS_nitrate_soluble_5wave.txt')
       OPEN (unit=37, file='SABS_nitrate_insoluble_5wave.txt')
c
       OPEN (unit=24, file='SEXT_nitrate_insoluble_6bands.txt')
       OPEN (unit=25, file='SSA_nitrate_insoluble_6bands.txt')
       OPEN (unit=26, file='G_nitrate_insoluble_6bands.txt')
c
c--initializing wavelengths
c
       DO Nwv=1, NwvmaxSW-1
         lambda_int(Nwv)=( lambda(Nwv)+lambda(Nwv+1) ) /2.
       ENDDO
c
       DO nb=1, nb_lambda
         lambda_int(NwvmaxSW-1+nb)=lambda_ref(nb)
       ENDDO
c
c
c---now start calculations 
c
       DO mode=1, Nmode
c
c--lecture des indices from an old file
       OPEN(unit=20,file='ri_nitrate_AS_v2')  !--same data for AS and CS
       DO IRH=1,rh_int      
       DO wv=1, nb_wv
        READ (20,*) rh_dummy, wv_nitrate(wv),
     .        index_r_nitrate(irh,mode,wv), index_i_nitrate(irh,mode,wv)
       ENDDO
       ENDDO
       CLOSE(20)
c
c--loop over RH
c
        DO IRH=1,rh_int      
c
        DO Nwv=1, NwvmaxSW-1+nb_lambda
c
c--sinon plus proche voisin
        n_r=index_r_nitrate(irh,mode,1)
        n_i=index_i_nitrate(irh,mode,1)
        DO wv=1, nb_wv
          IF (wv_nitrate(wv).LT.lambda_int(Nwv)) THEN
            n_r=index_r_nitrate(irh,mode,wv)
            n_i=index_i_nitrate(irh,mode,wv)
          ENDIF  
        ENDDO
c
        m=CMPLX(n_r,-n_i)
c
        sigma_sca=0.0
        sigma_ext=0.0
        sigma_abs=0.0
        gtot=0.0
        omegatot=0.0
        masse = 0.0
        volume=0.0
        surface=0.0
c
        DO bin=0, Nbin !---loop on size bins
 
c--here r is rdry
        r=exp(log(rmin)+FLOAT(bin)/FLOAT(Nbin)*(log(rmax)-log(rmin)))
c
        rwet=growth(IRH,mode)*r
c
        x=2.*pi*rwet/lambda_int(Nwv)
        deltar=1./FLOAT(Nbin)*(log(rmax)-log(rmin))
c
c--rwet r*1e2 to get into cm unit
c
        number=0
        DO dis=1, Ndis
          number=number+
     .       Ntot(dis,mode)/SQRT(2.*pi)/log(sigma_g(dis,mode))*
     .       exp(-0.5*(log(r/(r_0(dis,mode)))/
     .               log(sigma_g(dis,mode)))**2)
        ENDDO
c--dry aerosol mass
        masse=masse  +4./3.*pi*r**3*number*deltar*rho*1.E3          !--g/m3
        volume=volume+4./3.*pi*r**3*number*deltar
        surface=surface+4.*pi*r**2*number*deltar
c
        k2=m
        k3=CMPLX(1.0,0.0)

        z2=CMPLX(x,0.0)
        z1=m*z2
 
        IF (0.0.LE.x.AND.x.LE.8.) THEN
        Nmax=INT(x+4*x**(1./3.)+1.)+2
        ELSEIF (8..LT.x.AND.x.LT.4200.) THEN
        Nmax=INT(x+4.05*x**(1./3.)+2.)+1
        ELSEIF (4200..LE.x.AND.x.LE.20000.) THEN
        Nmax=INT(x+4*x**(1./3.)+2.)+1
        ELSE
        WRITE(10,*) 'x out of bound, x=', x
        STOP
        ENDIF

        Nstart=Nmax+10

C-----------loop for nu1z1, nu1z2

       nu1z1(Nstart)=CMPLX(0.0,0.0)
       nu1z2(Nstart)=CMPLX(0.0,0.0)
       DO n=Nstart-1, 1 , -1
       nn=CMPLX(FLOAT(n),0.0)
       nu1z1(n)=(nn+1.)/z1 - 1./( (nn+1.)/z1 + nu1z1(n+1) ) 
       nu1z2(n)=(nn+1.)/z2 - 1./( (nn+1.)/z2 + nu1z2(n+1) )
       ENDDO

C------------loop for nu3z2

       nu3z2(0)=-I
       DO n=1, Nmax
       nn=CMPLX(FLOAT(n),0.0)
       nu3z2(n)=-nn/z2 + 1./ (nn/z2 - nu3z2(n-1) ) 
       ENDDO

C-----------loop for psiz2 and ksiz2 (z2)
       ksiz2(-1)=COS(REAL(z2))-I*SIN(REAL(z2))
       ksiz2(0)=SIN(REAL(z2))+I*COS(REAL(z2))
       DO n=1,Nmax
       nn=CMPLX(FLOAT(n),0.0)
       ksiz2(n)=(2.*nn-1.)/z2 * ksiz2(n-1) - ksiz2(n-2)
       psiz2(n)=CMPLX(REAL(ksiz2(n)),0.0)
       ENDDO

C-----------loop for a(n) and b(n)
    
       DO n=1, Nmax
        u1=k3*nu1z1(n) - k2*nu1z2(n)
        u5=k3*nu1z1(n) - k2*nu3z2(n)
        u6=k2*nu1z1(n) - k3*nu1z2(n)
        u8=k2*nu1z1(n) - k3*nu3z2(n)
        a(n)=psiz2(n)/ksiz2(n) * u1/u5
        b(n)=psiz2(n)/ksiz2(n) * u6/u8
       ENDDO

C-----------------final loop--------------
        Q_ext=0.0
        Q_sca=0.0
        g=0.0
        DO n=Nmax-1,1,-1
          nnn=FLOAT(n)
          Q_ext=Q_ext+ (2.*nnn+1.) * REAL( a(n)+b(n) ) 
          Q_sca=Q_sca+ (2.*nnn+1.) * 
     .               REAL( a(n)*CONJG(a(n)) + b(n)*CONJG(b(n)) )
          g=g + nnn*(nnn+2.)/(nnn+1.) * 
     .       REAL( a(n)*CONJG(a(n+1))+b(n)*CONJG(b(n+1)) )  + 
     .          (2.*nnn+1.)/nnn/(nnn+1.) * REAL(a(n)*CONJG(b(n)))
        ENDDO
        Q_ext=2./x**2 * Q_ext
        Q_sca=2./x**2 * Q_sca
        Q_abs=Q_ext-Q_sca
        IF (AIMAG(m).EQ.0.0) Q_abs=0.0
        omega=Q_sca/Q_ext
        g=g*4./x**2/Q_sca
c
        sigma_sca=sigma_sca+rwet**2*Q_sca*number*deltar
        sigma_abs=sigma_abs+rwet**2*Q_abs*number*deltar
        sigma_ext=sigma_ext+rwet**2*Q_ext*number*deltar
        omegatot=omegatot+rwet**2*Q_ext*omega*number*deltar
        gtot    =gtot+rwet**2*Q_sca*g*number*deltar 
c
        ENDDO   !---bin
C------------------------------------------------------------------

        sigma_sca=pi*sigma_sca
        sigma_abs=pi*sigma_abs
        sigma_ext=pi*sigma_ext
        gtot=pi*gtot/sigma_sca
        omegatot=pi*omegatot/sigma_ext
c
        final_g(Nwv)=gtot
        final_w(Nwv)=omegatot
        final_a(Nwv)=sigma_ext/masse
c
        ENDDO  !--loop on wavelength
c
c---averaging over LMDZ wavebands
c
        DO band=1, NbandSW
          gcm_a(band)=0.0
          gcm_g(band)=0.0
          gcm_w(band)=0.0
          gcm_weight_a(band)=0.0
          gcm_weight_g(band)=0.0
          gcm_weight_w(band)=0.0
        ENDDO
c
c---band 1 is now in the UV, so we take the first wavelength as being representative 
c---it doesn't matter anyway because all radiation is absorbed in the stratosphere
       DO Nwv=1,1
          band=1
          gcm_a(band)=gcm_a(band)+final_a(Nwv)*weight(Nwv)
          gcm_weight_a(band)=gcm_weight_a(band)+weight(Nwv)
          gcm_w(band)=gcm_w(band)+
     .                final_w(Nwv)*final_a(Nwv)*weight(Nwv)
          gcm_weight_w(band)=gcm_weight_w(band)+
     .                       final_a(Nwv)*weight(Nwv)
          gcm_g(band)=gcm_g(band)+
     .                final_g(Nwv)*final_a(Nwv)*final_w(Nwv)*weight(Nwv)
          gcm_weight_g(band)=gcm_weight_g(band)+
     .                       final_a(Nwv)*final_w(Nwv)*weight(Nwv)
        ENDDO
c
       DO Nwv=1,NwvmaxSW-1
c
        IF (Nwv.LE.5) THEN          !--RRTM spectral interval 2
          band=2
        ELSEIF (Nwv.LE.10) THEN     !--RRTM spectral interval 3
          band=3
        ELSEIF (Nwv.LE.16) THEN     !--RRTM spectral interval 4
          band=4
        ELSEIF (Nwv.LE.21) THEN     !--RRTM spectral interval 5
          band=5
        ELSE                        !--RRTM spectral interval 6
          band=6
        ENDIF
c
        gcm_a(band)=gcm_a(band)+final_a(Nwv)*weight(Nwv)
        gcm_weight_a(band)=gcm_weight_a(band)+weight(Nwv)
        gcm_w(band)=gcm_w(band)+
     .              final_w(Nwv)*final_a(Nwv)*weight(Nwv)
        gcm_weight_w(band)=gcm_weight_w(band)+
     .                     final_a(Nwv)*weight(Nwv)
        gcm_g(band)=gcm_g(band)+
     .              final_g(Nwv)*final_a(Nwv)*final_w(Nwv)*weight(Nwv)
        gcm_weight_g(band)=gcm_weight_g(band)+
     .                     final_a(Nwv)*final_w(Nwv)*weight(Nwv)

        ENDDO
c
        DO band=1, NbandSW
          gcm_a(band)=gcm_a(band)/gcm_weight_a(band)
          gcm_w(band)=gcm_w(band)/gcm_weight_w(band)
          gcm_g(band)=gcm_g(band)/gcm_weight_g(band)
          ss_a(band,IRH)=gcm_a(band)
          ss_w(band,IRH)=gcm_w(band)
          ss_g(band,IRH)=gcm_g(band)
        ENDDO
c
        DO nb=NbandSW+1, NbandSW+nb_lambda
          ss_a(nb,IRH)=final_a(NwvmaxSW-1+nb-NbandSW)
          ss_w(nb,IRH)=final_w(NwvmaxSW-1+nb-NbandSW)
          ss_g(nb,IRH)=final_g(NwvmaxSW-1+nb-NbandSW)
        ENDDO
c
        ENDDO   !--fin boucle sur RH
c
c--Outputs soluble
C 
        WRITE(14,*) '  ! '//chmode(mode)
        DO k=1, NbandSW
          WRITE(14,951) (ss_a(k,IRH),IRH=1,rh_int)
        ENDDO
        WRITE(15,*) '  ! '//chmode(mode)
        DO k=1, NbandSW
          WRITE(15,951) (ss_w(k,IRH),IRH=1,rh_int)
        ENDDO
        WRITE(16,*) '  ! '//chmode(mode)
        DO k=1, NbandSW
          WRITE(16,951) (ss_g(k,IRH),IRH=1,rh_int)
        ENDDO
c
        WRITE(34,*) '  ! extinction '//chmode(mode)
        DO k=NbandSW+1,NbandSW+nb_lambda
        WRITE(34,951) (ss_a(k,IRH),IRH=1,rh_int)
        ENDDO
        WRITE(36,*) '  ! absorption '//chmode(mode)
        DO k=NbandSW+1,NbandSW+nb_lambda
        WRITE(36,951) 
     .    ((1.0-ss_w(k,IRH))*ss_a(k,IRH),IRH=1,rh_int)
        ENDDO
c
c--Outputs insoluble
C 
        IF (mode.EQ.1) THEN 
c
        WRITE(24,*) '  ! Nitrate insoluble'
        WRITE(24,953) (ss_a(k,1),k=1,NbandSW)
        WRITE(25,*) '  ! Nitrate insoluble'
        WRITE(25,953) (ss_w(k,1),k=1,NbandSW)
        WRITE(26,*) '  ! Nitrate insoluble'
        WRITE(26,953) (ss_g(k,1),k=1,NbandSW)
c
        WRITE(35,*) '  ! extinction Nitrate insoluble'
        WRITE(35,954) (ss_a(k,1),k=NbandSW+1,NbandSW+nb_lambda)
        WRITE(37,*) '  ! absorption Nitrate insoluble'
        WRITE(37,954)
     .     ((1.0-ss_w(k,1))*ss_a(k,1),k=NbandSW+1,NbandSW+nb_lambda)
c
        ENDIF
c
        ENDDO !--boucle sur les modes

951     FORMAT(1X,12(F6.3,','),' &')
953     FORMAT(1X,5(F6.3,','),F6.3,' /')
954     FORMAT(1X,4(F6.3,','),F6.3,' /')
c
101     FORMAT(F10.6,E13.6)
c
        CLOSE(14)
        CLOSE(15)
        CLOSE(16)
c
        CLOSE(24)
        CLOSE(25)
        CLOSE(26)
c
        CLOSE(34)
        CLOSE(35)
        CLOSE(36)
        CLOSE(37)
c
        END
